﻿using System;
using System.Collections.Generic;

namespace System.Web.Mvc
{
	/// <summary>
	/// Represents a validator which validates the defined element list against the given max length
	/// value.
	/// </summary>
	public class ValidateMaxLength : Validator
	{
		public int MaxLength { get; set; }

		/// <summary>
		/// Initializes a new instance of the ValidateMaxLength class with the given elements to validate.
		/// </summary>
		public ValidateMaxLength(string elementsToValidate)
			: base(elementsToValidate)
		{
		}

		/// <summary>
		/// Initializes a new instance of the ValidateMaxLength class with the given elements to validate
		/// and max length value.
		/// </summary>
		public ValidateMaxLength(string elementsToValidate, int maxLength)
			: base(elementsToValidate)
		{
			if(maxLength < 0)
				throw new ArgumentOutOfRangeException("maxLength");

			MaxLength = maxLength;
		}

		/// <summary>
		/// Gets the rule for the given element used by the jQuery validation plugin.
		/// </summary>
		public override string GetClientRule(string element)
		{
			return "maxlength:" + MaxLength;
		}

		/// <summary>
		/// Gets the message for the given element used by the jQuery validation plugin.
		/// </summary>
		public override string GetClientMessage(string element)
		{
			return string.Format("maxlength:'{0}'", GetLocalizedErrorMessage(element, MaxLength)).Replace("'", "\'");
		}

		/// <summary>
		/// Validates the given element using the Values collection and generates an error if 
		/// invalid.
		/// </summary>
		protected override void Validate(string element)
		{
			if(Values.ContainsKey(element) == false || (Values[element] ?? string.Empty).Trim().Length > MaxLength)
				InsertError(element, MaxLength);
		}

		/// <summary>
		/// Gets the default error message format in English and is called if no error message is 
		/// defined in code or in App_GlobalResources.
		/// </summary>
		protected override string GetDefaultErrorMessageFormat()
		{
			return "The field {0} must contain a value value no longer than {1} characters";
		}
	}
}
