﻿using System;
using System.Collections.Generic;

namespace System.Web.Mvc
{
	/// <summary>
	/// Represents a validator which validates the defined element list against the given max length 
	/// value.
	/// </summary>
	public class ValidateMinLength : Validator
	{
		public int MinLength { get; set; }

		/// <summary>
		/// Initializes a new instance of the ValidateMinLength class with the given elements to validate.
		/// </summary>
		public ValidateMinLength(string elementsToValidate)
			: base(elementsToValidate)
		{
		}

		/// <summary>
		/// Initializes a new instance of the ValidateMinLength class with the given elements to validate
		/// and min length value.
		/// </summary>
		public ValidateMinLength(string elementsToValidate, int minLength)
			: base(elementsToValidate)
		{
			if(minLength < 0)
				throw new ArgumentOutOfRangeException("minLength");

			MinLength = minLength;
		}

		/// <summary>
		/// Gets the rule for the given element used by the jQuery validation plugin.
		/// </summary>
		public override string GetClientRule(string element)
		{
			return "minlength:" + MinLength;
		}

		/// <summary>
		/// Gets the message for the given element used by the jQuery validation plugin.
		/// </summary>
		public override string GetClientMessage(string element)
		{
			return string.Format("minlength:'{0}'", GetLocalizedErrorMessage(element, MinLength)).Replace("'", "\'");
		}

		/// <summary>
		/// Validates the given element using the Values collection and generates an error if 
		/// invalid.
		/// </summary>
		protected override void Validate(string element)
		{
			if(Values.ContainsKey(element) == false || (Values[element] ?? string.Empty).Trim().Length < MinLength)
				InsertError(element, MinLength);
		}

		/// <summary>
		/// Gets the default error message format in English and is called if no error message is 
		/// defined in code or in App_GlobalResources.
		/// </summary>
		protected override string GetDefaultErrorMessageFormat()
		{
			return "The field {0} must contain a value of at least {1} characters";
		}
	}
}
